/*****************************************************************************
*** SPRDEM4B
*** Scrolling Part 2
*** April 25, 2000
*** Matthias Domin
***
*** This demo shows the creation of a 320*320 pixels wide playfield using
*** a "character"-set of 15 tiles cut out of a windows BMP-file and how to
*** scroll through this playfield using the voff and hoff registers.
***
*** Use the SPR.BAT batch-file to create the sprites-library out
*** of the BG.BMP. This BMP-file is 80*64 (w,h) pixel wide, but we only cut
*** 5*3 tiles of 16*16 pixels out of it.
***
*** In addition to SPRDEMO4.C we set up a status bar at the bottom of the screen
***
*****************************************************************************/

#include <lynx.h>
#include <lynxlib.h>
#include <stdlib.h>

// LYNX-specific #defines:
#define JOY_RIGHT  0x10
#define JOY_LEFT   0x20
#define JOY_DOWN   0x40
#define JOY_UP     0x80

#define BUTTON_OPTION1 0x08
#define BUTTON_OPTION2 0x04
#define BUTTON_INNER   0x02
#define BUTTON_OUTER   0x01

#define BUTTON_PAUSE   0x01

// program specific #defines:

// We need (TILES_X * TILE_WIDTH) * (TILES_Y * TILE_HEIGHT) pixels
// to cover the screen:
#define TILE_WIDTH  16
#define TILE_HEIGHT 16

// In either direction, we need one tile more SCREEN_WIDTH/TILE_WIDTH
// This means for the horizontal direction: 160/16 * 1 = 10 + 1 = 11
#define TILES_X 11
// Please observe, that 102/16 + 1 = 6.375 + 1 = 7.375 has to become 8!
// If you use only 7 tiles for this direction, you will see some debris
// on the last 6 lines of the screen! To avoid this, you have to cover
// the last 6 screenlines with another sprite.
// In SPRDEM4B.C we use the lower 16+6 screen line to show a status bar,
// so we set TILES_Y to 6
#define TILES_Y 6

// the colour palette:
#include ".\sprites\bg.pal"

extern uchar spr[TILES_Y][TILES_X][11];
extern uchar init_scb[23];
extern uchar spr0[11];
extern char* Sprites[];

#asm
          xref _bg000000, _bg000001, _bg000002, _bg000003, _bg000004
          xref _bg001000, _bg001001, _bg001002, _bg001003, _bg001004
          xref _bg002000, _bg002001, _bg002002, _bg002003, _bg002004

; Address of the array for the 8*11 daisychained sprites:
; It's below the first screen which starts at $a000.
; We need 8*11*11 = 968 (0x03c8) bytes
; Always be carefull when doing such things!
_spr = $9c00    ; $a000 - 0x03c8 ~= $9c00

; We use init_scb to set up the SpriteEngine registers with the global
; Size- and Pen-values.
; Using this trick, the 8*11 spr[]-SCBs only need 11 Bytes each!
_init_scb:
          dc.b $c1, $10, $20
          dc.w _spr          ; PTR_NEXT points to the first visible sprite
          dc.w dummy         ; PTR_DATA points to a NULL-sprite
          dc.w 200, 200      ; x,y outside the visible screen area
          dc.w $100, $100    ; all sprites will be scaled to normal size
          ; all sprites will use these pen-index-table:
          dc.b $01, $23, $45, $67, $89, $ab, $cd, $ef

dummy:    dc.b 0,0,0  ; No real sprite :)

; spr0 is used as a template for the 8*11 chained sprites
_spr0:    dc.b $c1, $08, $00 ; CTL0, CTL1 = reuse existing palette, COLL
          dc.w 0, _bg000001  ; pDataNext, pData
          dc.w 0, 0          ; x, y

; the table with pointers to the 15 created tile-sprites:
_Sprites: dc.w _bg000000, _bg000001, _bg000002, _bg000003, _bg000004
          dc.w _bg001000, _bg001001, _bg001002, _bg001003, _bg001004
          dc.w _bg002000, _bg002001, _bg002002, _bg002003, _bg002004
#endasm


// Here comes the definition of our world: 20*20 tiles
SpriteMap[20][20] =
 {
 {14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14},
 {14, 0,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13, 1,14},
 {14,10, 2,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12, 6,14},
 {14,10,11, 4, 0,13,13,13,13, 1,14,14,14,14,14,14,14,14,14,14},
 {14,10,11, 4,10, 2,12,12, 3,11,14, 4, 4, 4, 4, 4, 4, 4, 4, 4},
 {14,10, 7,13, 8,11,14,14,10,11,14, 4, 4, 4, 4, 4, 4, 4, 4, 4},
 {14, 5,12,12,12, 6,14,14,10,11,14, 4, 4, 4, 4, 4, 4, 4, 4, 4},
 {14,14,14,14,14,14,14,14,10,11,14, 4, 4, 4, 4, 4, 4, 4, 4, 4},
 { 4, 4, 4, 4, 4, 4, 4,14,10,11,14, 4, 4, 4, 4, 4, 4, 4, 4, 4},
 { 4, 4, 4, 4, 4, 4, 4,14,10,11,14, 4, 4, 4, 4, 4, 4, 4, 4, 4},
 { 4, 4, 4, 4, 4, 4, 4,14,10,11,14, 4, 4, 4, 4, 4, 4, 4, 4, 4},
 { 4, 4, 4, 4, 4, 4, 4,14,10,11,14, 4, 4, 4, 4, 4, 4, 4, 4, 4},
 { 4, 4, 4, 4, 4, 4, 4,14,10,11,14, 4, 4, 4, 4, 4, 4, 4, 4, 4},
 { 4, 4, 4, 4, 4, 4, 4,14,10,11,14, 4, 4, 4, 4, 4, 4, 4, 4, 4},
 { 4, 4, 4, 4, 4, 4, 4,14,10,11,14, 4, 4, 4, 4, 4, 4, 4, 4, 4},
 { 4, 4, 4, 4, 4, 4, 4,14,10,11,14, 4, 4, 4, 4, 4, 4, 4, 4, 4},
 { 4, 4, 4, 4, 4, 4, 4,14,10,11,14, 4, 4, 4, 4, 4, 4, 4, 4, 4},
 { 4, 4, 4, 4, 4, 4, 4,14,10,11,14, 4, 4, 4, 4, 4, 4, 4, 4, 4},
 {14, 4, 4, 4, 4, 4, 4,14, 5, 6,14, 4, 4, 4, 4, 4, 4, 4, 4,14},
 {14,14, 4, 4, 4, 4, 4,14,14,14,14, 4, 4, 4, 4, 4, 4, 4,14,14}
 };


register uchar tile_col, tile_row;
register uchar tile_x, tile_y;

char szTemp[20+1];      // for text messages
char szText1[] = "SPRDEM4B";
char szText2[] = "by M.Domin";


/*****************************************************/
extern uchar VBLflag;
#asm
_VBLflag = $a0
#endasm

#define VSYNC {++VBLflag;while( VBLflag );}

VBL() interrupt
{
   VBLflag = 0; /* indicates that a VBL has ocurred */
}

HBL() interrupt
{
}

/*****************************************************/

/****************************************/
/* Converts an uint into a ASCII-string */
/****************************************/
UintToASCII(uNr, pStr)
uint uNr;
char * pStr;
{
   strcpy(pStr, "00000");
   while(uNr >= 10000)
      {
      uNr -= 10000;
      (*pStr)++;
      }
   pStr++;
   while(uNr >= 1000)
      {
      uNr -= 1000;
      (*pStr)++;
      }
   pStr++;
   while(uNr >= 100)
      {
      uNr -= 100;
      (*pStr)++;
      }
   pStr++;
   while(uNr >= 10)
      {
      uNr -= 10;
      (*pStr)++;
      }
   pStr++;
   *pStr += (uint) uNr;
}


/****************************************/
/* Paints the screen                    */
/****************************************/
ShowScreen()
{
register uchar row,col;

   // Clear screen: Not necessary, because we are covering the whole screen!
   // DrawFBox(hoff,voff,160,102, 1);

   // Set up the 8*11 screen sprites
   for (row = 0; row < TILES_Y; ++row)
      for (col = 0; col < TILES_X; ++col)
         SCBDATA(&spr[row][col][0]) = Sprites[SpriteMap[tile_row+row][tile_col+col]];

   // And now, draw the sprites:
   DrawSprite(init_scb);

   // SPRDEM4B.C: Cover the last 16+6 screen lines with a status bar
   DrawFBox(hoff,voff+102-(16+6),160,16+6, 1);
   // Method2: Set hoff and voff to 0:
   // Method2: hoff = voff = 0;
   // Method2: DrawFBox(0,102-(16+6),160,16+6, 1);


   // Still no sprintf-function?
   UintToASCII(tile_col, szTemp);
   szTemp[0] = 'X';
   szTemp[1] = '=';
   TextOut2(hoff,voff+102-(16+6)+2, 15, 0, szTemp);
   // Method2: TextOut2(0,102-(16+6)+2, 15, 0, szTemp);
   UintToASCII(tile_x, szTemp);
   szTemp[0] = ',';
   szTemp[1] = ' ';
   TextOut2(hoff+5*8,voff+102-(16+6)+2, 15, 0, szTemp);
   // Method2: TextOut2(5*8,102-(16+6)+2, 15, 0, szTemp);

   UintToASCII(tile_row, szTemp);
   szTemp[0] = 'Y';
   szTemp[1] = '=';
   TextOut2(hoff,voff+102-(16+6)+2+8+2, 15, 0, szTemp);
   // Method2: TextOut2(0,102-(16+6)+2+8+2, 15, 0, szTemp);
   UintToASCII(tile_y, szTemp);
   szTemp[0] = ',';
   szTemp[1] = ' ';
   TextOut2(hoff+5*8,voff+102-(16+6)+2+8+2, 15, 0, szTemp);
   // Method2: TextOut2(5*8,102-(16+6)+2+8+2, 15, 0, szTemp);


   TextOut2(hoff+12*8,voff+102-(16+6)+2, 14, 0, szText1);
   // Method2: TextOut2(12*8,102-(16+6)+2, 14, 0, szText1);
   TextOut2(hoff+10*8,voff+102-(16+6)+2+8+2, 14, 0, szText2);
   // Method2: TextOut2(10*8,102-(16+6)+2+8+2, 14, 0, szText2);

}

main()
{
register uchar x,y,c;
register uchar joy;

   InitIRQ();
   InstallIRQ(2,VBL);

   EnableIRQ(2);
   CLI;


   SetBuffers(0xa000, 0xc000, 0);

   _SetRGB(pal);


   // Set up the chain of 8*11 screen sprites
   for (y = 0; y < TILES_Y; ++y)
      for (x = 0; x < TILES_X; ++x)
         {
         bcopy(&spr0[0], &spr[y][x][0], 11);  // src, dest, count

         SCBX(&spr[y][x][0]) = 16*x;
         SCBY(&spr[y][x][0]) = 16*y;
   
         SCBNEXT(&spr[y][x][0]) = &spr[y][x][0] + 11;
         }
   SCBNEXT(&spr[TILES_Y-1][TILES_X-1][0]) = 0;	// Terminate the chain!!!


   tile_x = tile_y = 0;
   tile_col = tile_row = 0;

   // Draw first screen
   ShowScreen();

   //  main loop
   for(;;)
      {
      VSYNC;
      SwapBuffers();
      voff = tile_y;              // Set scroll offsets
      hoff = tile_x;              // Set scroll offsets

      ShowScreen();

      while (!(joy = joystick))   // Wait for user-input
         ;

      if (joy & JOY_LEFT)
         {
         // Will the new position still be inside the current tile?
         if (tile_x > 0) // Yes!
            {
            --tile_x;
            }
         else            // No!
            {
            // But can we go any further to this direction?
            if (tile_col > 0)  // Yes!
               {
               tile_x = TILE_WIDTH-1;
               --tile_col;
               }
            }
         }

      if (joy & JOY_RIGHT)
         {
         if (tile_x < TILE_WIDTH-1)
            {
            ++tile_x;
            }
         else
            {
            if (tile_col < 20-TILES_X)
               {
               tile_x = 0;
               ++tile_col;
               }
            }
         }

      if (joy & JOY_UP)
         {
         if (tile_y > 0)
            {
            --tile_y;
            }
         else
            {
            if (tile_row > 0)
               {
               tile_y = TILE_HEIGHT-1;
               --tile_row;
               }
            }
         }

      if (joy & JOY_DOWN)
         {
         if (tile_y < TILE_HEIGHT-1)
            {
            ++tile_y;
            }
         else
            {
            if (tile_row < 20-TILES_Y)
               {
               tile_y = 0;
               ++tile_row;
               }
            }
         }
      }  // forever
} // main
